import io
import os
import tempfile

import boto3
import numpy as np
from aicsimageio.writers import OmeTiffWriter
from PIL import Image

from io_collection.save.save_buffer import save_buffer_to_s3


def save_image(location: str, key: str, image: np.ndarray) -> None:
    if location[:5] == "s3://":
        save_image_to_s3(location[5:], key, image)
    else:
        save_image_to_fs(location, key, image)


def save_image_to_fs(path: str, key: str, image: np.ndarray) -> None:
    full_path = os.path.join(path, key)
    os.makedirs(os.path.split(full_path)[0], exist_ok=True)

    if key.endswith(".ome.tiff") or key.endswith(".ome.tif"):
        OmeTiffWriter.save(image, full_path)
    elif key.endswith(".png"):
        Image.fromarray(image).save(full_path)


def save_image_to_s3(bucket: str, key: str, image: np.ndarray) -> None:
    s3_client = boto3.client("s3")

    if key.endswith(".ome.tiff") or key.endswith(".ome.tif"):
        with tempfile.TemporaryDirectory() as temp_dir:
            temp_path = os.path.join(temp_dir, "temp.ome.tiff")
            OmeTiffWriter.save(image, temp_path)

            with open(temp_path, "rb") as fileobj:
                s3_client.upload_fileobj(fileobj, bucket, key)
    elif key.endswith(".png"):
        with io.BytesIO() as buffer:
            Image.fromarray(image).save(buffer, format="png")
            save_buffer_to_s3(bucket, key, buffer)



import io
import pickle
from typing import Any

from io_collection.save.save_buffer import save_buffer


def save_pickle(location: str, key: str, obj: Any) -> None:
    with io.BytesIO() as buffer:
        pickle.dump(obj, buffer)
        save_buffer(location, key, buffer)



import os
import shutil

import boto3


def copy_key(location: str, old_key: str, new_key: str) -> None:
    if location[:5] == "s3://":
        copy_key_on_s3(location[5:], old_key, new_key)
    else:
        copy_key_on_fs(location, old_key, new_key)


def copy_key_on_fs(path: str, old_key: str, new_key: str) -> None:
    full_old_path = os.path.join(path, old_key)
    full_new_path = os.path.join(path, new_key)
    os.makedirs(os.path.split(full_new_path)[0], exist_ok=True)
    shutil.copyfile(full_old_path, full_new_path)


def copy_key_on_s3(bucket: str, old_key: str, new_key: str) -> None:
    s3_client = boto3.client("s3")
    s3_client.copy_object(Bucket=bucket, CopySource=f"{bucket}/{old_key}", Key=new_key)



import os

import boto3


def remove_key(location: str, key: str) -> None:
    if location[:5] == "s3://":
        remove_key_on_s3(location[5:], key)
    else:
        remove_key_on_fs(location, key)


def remove_key_on_fs(path: str, key: str) -> None:
    full_path = os.path.join(path, key)
    os.remove(full_path)


def remove_key_on_s3(bucket: str, key: str) -> None:
    s3_client = boto3.client("s3")
    s3_client.delete_object(Bucket=bucket, Key=key)



import io
import os
from typing import Any

import matplotlib.figure as mpl

from io_collection.save.save_buffer import save_buffer_to_s3


def save_figure(location: str, key: str, figure: mpl.Figure, **kwargs: Any) -> None:
    if location[:5] == "s3://":
        save_figure_to_s3(location[5:], key, figure, **kwargs)
    else:
        save_figure_to_fs(location, key, figure, **kwargs)


def save_figure_to_fs(path: str, key: str, figure: mpl.Figure, **kwargs: Any) -> None:
    full_path = os.path.join(path, key)
    os.makedirs(os.path.split(full_path)[0], exist_ok=True)
    figure.savefig(full_path, **kwargs)


def save_figure_to_s3(bucket: str, key: str, figure: mpl.Figure, **kwargs: Any) -> None:
    """
    Saves figure to bucket with given key.
    """
    with io.BytesIO() as buffer:
        figure.savefig(buffer, **kwargs)
        save_buffer_to_s3(bucket, key, buffer)



import pickle
from typing import Any

from io_collection.load.load_buffer import load_buffer


def load_pickle(location: str, key: str) -> Any:
    buffer = load_buffer(location, key)
    return pickle.loads(buffer.read())



import io

from PIL import Image

from io_collection.load.load_buffer import load_buffer
from io_collection.save.save_buffer import save_buffer


def save_gif(location: str, key: str, frame_keys: list[str]) -> None:
    with io.BytesIO() as buffer:
        frames = [Image.open(load_buffer(location, frame_key)) for frame_key in frame_keys]
        frames[0].save(
            buffer, format="gif", save_all=True, append_images=frames[1:], duration=100, loop=0
        )
        save_buffer(location, key, buffer)



import json
import re
from typing import Union

from io_collection.save.save_text import save_text


def save_json(location: str, key: str, obj: Union[dict, list], levels: int = 5) -> None:
    contents = json.dumps(obj, indent=2)

    for level in range(1, levels + 1):
        match_pattern = r"\[" + ",".join([r"\n\s*([e\-\d\.]+)" for i in range(level)]) + r"\n\s*\]"
        sub_pattern = "[" + ",".join([f"\\{i + 1}" for i in range(level)]) + "]"
        pattern = re.compile(match_pattern)
        contents = pattern.sub(sub_pattern, contents)

    save_text(location, key, contents)



import os
from typing import Optional

from aicsimageio import AICSImage
from aicsimageio.readers import OmeTiffReader, TiffReader


def load_image(location: str, key: str, dim_order: Optional[str] = None) -> AICSImage:
    if location[:5] == "s3://":
        return load_image_from_s3(location[5:], key, dim_order)
    else:
        return load_image_from_fs(location, key, dim_order)


def load_image_from_fs(path: str, key: str, dim_order: Optional[str] = None) -> AICSImage:
    full_path = os.path.join(path, key)
    return AICSImage(
        full_path,
        reader=OmeTiffReader if key.endswith(".ome.tiff") else TiffReader,
        dim_order=dim_order,
    )


def load_image_from_s3(bucket: str, key: str, dim_order: Optional[str] = None) -> AICSImage:
    full_key = f"s3://{bucket}/{key}"
    return AICSImage(
        full_key,
        reader=OmeTiffReader if key.endswith(".ome.tiff") else TiffReader,
        dim_order=dim_order,
    )
